#!/usr/bin/ruby

require 'nkf'
require 'fileutils'
require 'date'


class IEPGdat
  attr_reader :content_type, :version
  attr_reader :station, :year, :month, :date
  attr_reader :start, :end, :program_title, :cache_path

  def initialize
    @content_type = "Content-type: application/x-tv-program-info; charset=shift_jis\n"
    @version = "version: 1\n"
    @station = "station: "
    @year = "year: "
    @month = "month: "
    @date = "date: "
    @start = "start: "
    @end = "end: "
    @program_title = "program-title: "
    @cache_path = ""
  end

  def read_iepg_data_from_file(file)
    @cache_path = file.dup
    File::open(file) {|fp|
        while l = fp.gets
          if l.slice(0, @station.length) == @station then
            @station = l
          elsif l.slice(0, @year.length) == @year then
            @year = l
          elsif l.slice(0, @month.length) == @month then
            @month = l
          elsif l.slice(0, @date.length) == @date then
            @date = l
          elsif l.slice(0, @start.length) == @start then
            @start = l
          elsif l.slice(0, @end.length) == @end then
            @end = l
          elsif l.slice(0, @program_title.length) == @program_title then
            @program_title = l
          end
        end
    }
  end

  def set_iepg_station_data(str)
    @station << str << "\n"
  end

  def set_iepg_year_data(str)
    @year << str << "\n"
  end

  def set_iepg_month_data(str)
    @month << str << "\n"
  end

  def set_iepg_date_data(str)
    @date << str << "\n"
  end

  def set_iepg_start_data(str)
    if str[3] != ':' then
      @start << str.slice(0..1) << ':' << str.slice(2..3) << "\n"
    else
       @start << str << "\n"
    end
  end

  def set_iepg_end_data(str)
    if str[3] != ':' then
      @end << str.slice(0..1) << ':' << str.slice(2..3) << "\n"
    else
       @end << str << "\n"
    end
  end

  def set_iepg_program_title_data(str)
    @program_title << str << "\n"
  end

  def set_iepg_data(station, year, month, date, start, ed, program_title)
    set_iepg_station_data(station)
    set_iepg_year_data(year)
    set_iepg_month_data(month)
    set_iepg_date_data(date)
    set_iepg_start_data(start)
    set_iepg_end_data(ed)
    set_iepg_program_title_data(program_title)
  end

  def output_iepg_data(output_dir)
    c_year = @year.scan(/[0-9]/)
    c_month = @month.scan(/[0-9]/)
    c_date = @date.scan(/[0-9]/)
    c_start = @start.scan(/[0-9]/)
    c_end = @end.scan(/[0-9]/)
    file_name = c_year.to_s + c_month.to_s + c_date.to_s + "_" + 
      c_start.to_s + "-" + c_end.to_s + ".tvpi"
    file_path = File.expand_path(file_name, output_dir)
    file = File.open(file_path,'w')
    file.print NKF.nkf('--windows', @content_type)
    file.print NKF.nkf('--windows', @version)
    file.print NKF.nkf('--windows', @station)
    file.print NKF.nkf('--windows', @year)
    file.print NKF.nkf('--windows', @month)
    file.print NKF.nkf('--windows', @date)
    file.print NKF.nkf('--windows', @start)
    file.print NKF.nkf('--windows', @end)
    file.print NKF.nkf('--windows', @program_title)
    file.close
    return file_path
  end

end #IEPGdat
