#!/usr/bin/ruby

require 'rubygems'
gem 'hpricot', '>=0.4'
require 'hpricot'
require 'open-uri'
require 'nkf'
require 'fileutils'
require 'date'

class RTVBookLib
  attr_reader :tvprogram_search_word

  def initialize(conf = nil)
    @dir = File::dirname(__FILE__)
    conf_path = conf || File.expand_path("rtvbook.conf", @dir)
    @conf_dir = File::dirname(conf_path)
    eval(File::open(conf_path) {|f| f.read }.untaint)

    cache_dir_path = File.expand_path('cache', @dir)
    unless File.exists?(cache_dir_path)
      FileUtils.mkdir(cache_dir_path)
    end
    @cache_dir = cache_dir_path

    @tvprogramlist_path = File.expand_path('tvprogramlist.xml', @dir)
    @syobocal_path = File.expand_path('syobocal.rdf', @dir)

#    @tv_grab_jp_path ||= "~/bin/tv_grab_jp"
#    @tv_grab_jp_days ||= 3
#    @use_tv_grab_jp ||= true
#    @use_syobocal ||= true
#    @syobocal_user_id ||= "rtvbook"
#    @syobocal_days ||= 3
#    @syobocal_titlefmt ||= "$(Mark)$(Title)"
    @channel_ids = [[]]
    @ontv_data = nil
    @iepg_cached = []
    @iepg_store = []
    @iepg_store_count = 0
  end

  def delete_used_iepg_cached_data
    # 開始時刻が今より古い cache ファイルを削除する。
    # 比較にはファイル名を使っている。
    nowtimestrings = Time.now.strftime("%Y%m%d_%H%M")
    cache_files = Dir::entries(@cache_dir)
    existing_cached_iepg_data_files = []
    cache_files.each do |c_file|
      if c_file.length > 13 then
        if c_file.slice(0,13) < nowtimestrings.slice(0,13) then
          FileUtils.rm(File.expand_path(c_file, @cache_dir))
          print "rtvbooklib.rb cache: deleted #{c_file}\n"
        else
          existing_cached_iepg_data_files << File.expand_path(c_file, @cache_dir)
        end
      end
    end
    load_existing_cached_iepg_data(existing_cached_iepg_data_files)
  end

  def load_existing_cached_iepg_data(files)
    n = 0
    files.each do |f|
      @iepg_cached[n] = IEPGdat::new
      @iepg_cached[n].read_iepg_data_from_file(f)
      n = n + 1
    end
  end

  def output_cache_file    
    @iepg_store.each do |iepg|
      fp = iepg.output_iepg_data(@cache_dir)
      print "rtvbooklib.rb cache: created #{fp}.\n"
    end
    if @iepg_store_count == 0 then
      print "rtvbooklib.rb cache: created nothing.\n"
    end
  end

  def feature_days_check(start_elem)
    # start_elem: YYYYMMDDhhmm
    # 現在時より未来のデータを参照しているかどうかを確認する。
    # true で現在時より先。false だと現在時より古い。
    nowtimestrings = Time.now.strftime("%Y%m%d%H%M")
    if nowtimestrings.to_i < start_elem.to_i then
      return true
    else
      return false
    end
  end
  
  def add_iepg_data_list(start_elem, stop_elem, station, program_title)
    # start_elem, stop_elem: YYYYMMDDhhmm
    # station, program_title: strings
    @iepg_store[@iepg_store_count] = IEPGdat::new    
    year = start_elem.slice(0..3)
    month = start_elem.slice(4..5)
    date = start_elem.slice(6..7)
    start = start_elem.slice(8..11)
    stop = stop_elem.slice(8..11)

    @iepg_store[@iepg_store_count].set_iepg_data(station,
      year, month, date, start, stop, program_title)
    print "rtvbooklib.rb cache: add #{program_title}, start #{start_elem}\n"
    @iepg_store_count = @iepg_store_count + 1
  end

  def compare_already_cached_iepg_data(start_elem, stop_elem, title)
    # start_elem, stop_elem: YYYYMMDDhhmm
    # title: strings
    # 既に cache ファイル作成済みかどうかを確認する
    # 開始時間もしくは終了時間が違う場合は cache を削除する。
    # cache ありで true、cache なしで false を返す
    @iepg_cached.each do |c|
      c_year = c.year.scan(/[0-9]/)
      c_month = c.month.scan(/[0-9]/)
      c_date = c.date.scan(/[0-9]/)
      c_start = c.start.scan(/[0-9]/)
      c_end = c.end.scan(/[0-9]/)
      a_start_elem = c_year + c_month + c_date + c_start
      a_stop_elem = c_year + c_month + c_date + c_end
      c_start_elem = String::new
      c_stop_elem = String::new
      a_start_elem.each do |char|
        c_start_elem << char
      end
      a_stop_elem.each do |char|
        c_stop_elem << char
      end

      c_title = NKF.nkf('-Lu -d -w', c.program_title)
      c_title = c_title.slice(15, c_title.length - 15)
      c_title = c_title.chomp

      if c_title == title then
        if c_start_elem == start_elem && c_stop_elem == stop_elem then
          return true
        end
        # 同一タイトルで日付同じで時間が違う cache も捨てる。(基準は適当……)
        if c_start_elem.slice(0,8) == start_elem.slice(0,8) then
          FileUtils.rm(c.cache_path)
          print "rtvbooklib.rb cache: time changed. delete #{c.cache_path}\n"
        end
      end
    end
    return false
  end

  def update_tvprogramlist
    result = system("#{@tv_grab_jp_path} --days #{@tv_grab_jp_days} --output #{@tvprogramlist_path}") if @use_tv_grab_jp
    
    if @use_syobocal then
        file = File.open(@syobocal_path, 'w')
      file.print(open("http://cal.syoboi.jp/rss.php?usr=#{@syobocal_user_id}&days=#{@syobocal_days}&titlefmt=#{@syobocal_titlefmt}").read)
        file.close
    end
  end

  def search_word_matching
    search_word_matching_ontv if @use_tv_grab_jp
    search_word_matching_syobocal if @use_syobocal
  end

  #---------------------------------------------------------------------#
  # 以下はしょぼいカレンダー使用時の method

  def search_word_matching_syobocal
    # ワードサーチし、マッチングした場合は放送局も合っているか照合する

    @syobocal_data = Hpricot.XML(open(@syobocal_path))

    @tvprogram_search_word.each do |word|
      (@syobocal_data/"title[text()*='#{word[0]}']").each do |t|
        if (t.parent/"dc:publisher").inner_text == word[1] &&
            feature_days_check_syobocal(t.parent) == true then
          if compare_already_cached_iepg_data_sybocal(t.parent, word) != true then
            add_iepg_data_list_syobocal(t.parent, word)
          end
        end
      end
    end
  end

  def syobocal_timestrings_convert(s)
    return s.slice(0..3) + s.slice(5..6) + s.slice(8..9) + s.slice(11..12) + s.slice(14..15)
  end

  def feature_days_check_syobocal(hdoc)
    str = (hdoc/"tv:startDatetime").inner_text
    start_elem = syobocal_timestrings_convert(str)
    return feature_days_check(start_elem)
  end

  def compare_already_cached_iepg_data_sybocal(hdoc, word)
    str1 = (hdoc/"tv:startDatetime").inner_text
    start_elem = syobocal_timestrings_convert(str1)
    str2 = (hdoc/"tv:endDatetime").inner_text
    stop_elem = syobocal_timestrings_convert(str2)
    title = (hdoc/"title[text()*='#{word[0]}']").inner_text
    return compare_already_cached_iepg_data(start_elem, stop_elem, title)
  end

  def add_iepg_data_list_syobocal(hdoc, word)
    str1 = (hdoc/"tv:startDatetime").inner_text
    start_elem = syobocal_timestrings_convert(str1)
    str2 = (hdoc/"tv:endDatetime").inner_text
    stop_elem = syobocal_timestrings_convert(str2)
    program_title = (hdoc/"title[text()*='#{word[0]}']").inner_text
    station = word[1].dup
    add_iepg_data_list(start_elem, stop_elem, station, program_title)
  end

  # ここまでがしょぼいカレンダー使用時の method
  #---------------------------------------------------------------------#

  #---------------------------------------------------------------------#
  # 以下は tv_grab_jp 使用時の method

  def create_channel_list(hdoc)
    # tv_grab_jp で取得した放送局の名前と id を保持する。
    ch_name = []
    ch_id = []
    n = 0
    (hdoc/"channel").each do |channel|
      ch_name[n] = (channel/'display-name [@lang="ja_JP"]').inner_html
      ch_id[n] = (channel[:id]).to_s
      n = n + 1
    end
    return ch_id.zip(ch_name)
  end

  def search_word_matching_ontv
    # ワードサーチし、マッチングした場合は放送局も合っているか照合する

    @ontv_data = Hpricot.XML(open(@tvprogramlist_path))

    @channel_ids = create_channel_list(@ontv_data)

    @tvprogram_search_word.each do |word|
      (@ontv_data/"title[text()*='#{word[0]}']").each do |t|
        @channel_ids.each do |ch|
          if ch[1] == word[1] then
            if (t.parent[:channel]).to_s == ch[0] &&
               feature_days_check_ontv(t.parent) == true then
              if compare_already_cached_iepg_data_ontv(t.parent, word) != true then
                add_iepg_data_list_ontv(t.parent, word)
              end
            end
          end
        end
      end
    end
  end

  def feature_days_check_ontv(hdoc)
    start_elem = (hdoc[:start]).slice(0..11)
    return feature_days_check(start_elem)
  end

  def add_iepg_data_list_ontv(hdoc, word)
    start_elem = (hdoc[:start]).slice(0..11)
    stop_elem = (hdoc[:stop]).slice(0..11)
    station = word[1].dup
    program_title = (hdoc/"title[text()*='#{word[0]}']").inner_text
    add_iepg_data_list(start_elem, stop_elem, station, program_title)
  end

  def compare_already_cached_iepg_data_ontv(hdoc, word)
    start_elem = (hdoc[:start]).slice(0..11)
    stop_elem = (hdoc[:stop]).slice(0..11)
    title = (hdoc/"title[text()*='#{word[0]}']").inner_text
    return compare_already_cached_iepg_data(start_elem, stop_elem, title)
  end
    
  # ここまでが tv_grab_jp 使用時の method
  #---------------------------------------------------------------------#

end #RTVBookLib
